/*
 *	file.h		File-related declarations and such.
 *
 */

#ifndef LOCKD_FILE_H
#define LOCKD_FILE_H

#include "nlm_prot.h"

/*
 * This describes a file subject to locking. This struct is used
 * only on the server side of things.
 */
struct lockd_file {
    dev_t		i_dev;
    ino_t		i_inum;
    unsigned int	i_version;
    netobj		fh;
    int			fd;

    lockd_lock *	granted;		/* granted locks */
    lockd_lock *	blocked;		/* Queue of blocked reqs. */
    lockd_lock *	acknowledged;		/* formerly blocked, GRANT
						 * notification sent.
						 */

    lockd_file *	nxt;			/* Doubly linked list */
    lockd_file *	prv;

    unsigned short	retry_blocked : 1,
			update_locks  : 1,
			read_only     : 1;

    char *		name;
};

lockd_file *	file_lookup(lockd_host *host, netobj *nfsfh, int local);
lockd_file *	file_create(lockd_host *host, netobj *nfsfh, int local);
void		file_release(lockd_host *host, lockd_file *file);
void		file_delete(lockd_host *host, lockd_file * file);
void		file_init(void);
int		file_filedesc(lockd_file * file);
char *		file_pathname(lockd_file * file);
int		file_fd(lockd_file *file);
int		file_getversion(int fd);
void		file_foreach(void (*func)(lockd_file *));
void		file_retry_blocked(lockd_file *file);

#endif /* LOCKD_FILE_H */
