/*
 *	host.h		Host-related declarations and prototypes.
 *
 */

#ifndef LOCKD_HOST_H
#define LOCKD_HOST_H

/*
 * This describes a host that has requested a lock or that we have
 * sent a request to.
 */
struct lockd_host {
    char *		name;

    lockd_host *	nxt;
    lockd_host *	prv;

    lockd_file *	file_tbl;

					/* Used by the server side: */
    lockd_lock *	r_granted;	/* locks held by host */
    lockd_lock *	r_blocked;	/* blocked lock reqs from remote */

					/* Used by the client side: */
    lockd_lock *	l_granted;	/* locks we hold on that host */
    lockd_req *		l_blocked;	/* blocked locks for remote */
    lockd_req *		l_reclaim;	/* requests being reclaimed */
    lockd_req *		l_waiting;	/* pending requests */

    struct in_addr	addr;		/* primary address (for RPC calls) */
    struct in_addr	*more_addrs;	/* NULL for single-homed hosts */
    unsigned int	nr_addrs;
    int			use_msg;	/* use message passing (unused now) */

    int			monitored;
    int			state;
};

void		host_init(void);
lockd_host *	host_lookup(struct svc_req *rqstp, char *owner, int auth);
lockd_host *	host_create(struct svc_req *rqstp, char *owner, int auth);
void		host_monitor(lockd_host *host, int state);
void		host_unmonitor(lockd_host *host);
void		host_delete(lockd_host *pc);
void		host_block_clntreq(lockd_lock *lock);
void		host_cancel_clntreq(lockd_lock *lock);
char *		host_myname(void);

#endif /* LOCKD_HOST_H */
