/*
 * lklm.h	This is my own little version of a kernel-to-daemon
 *		lock communication protocol. I call it LKLM (L as in
 *		Linux). The biggest diffeence in contrast to the 
 *		original KLM is that it does not use RPC, and runs 
 *		over a UNIX domain socketpair.
 *
 *		Currently, it's restricted to POSIX-style locks because
 *		BSD flock-style locks are close to impossible. Other
 *		systems do not seem to support flock() over NLM either...
 *
 *							--okir
 */

#ifndef _LKLM_H_
#define _LKLM_H_


#ifdef DEBUG
#define _PATH_LKLMSOCK		"/tmp/lklmsock"
#endif

/*
 * Unfortunately, the linux header files don't declare SIGLOST (it's
 * aliased to SIGIO and commented out.
 */
#include <signal.h>
#ifndef SIGLOST
#define SIGLOST		SIGIO
#endif

struct lklm_lock {
	char			server_name[256];
	struct {
		int		len;
		unsigned char	data[64];	/* enough for NFSv2 and v3 */
	}			fh;
	int			pid;
	loff_t			l_offset;
	loff_t			l_len;
};

struct lklm_testarg {
	struct lklm_lock	alock;
	char			exclusive;
};

struct lklm_lockarg {
	struct lklm_lock	alock;		/* lock data */
	char			exclusive;
	char			block;
};

struct lklm_unlockarg {
	struct lklm_lock	alock;
};

struct lklm_holder {
	struct lklm_lock	alock;
	char			exclusive;
};

struct lklm_request {
	int			prot_version;
	int			lock_proto;
	int			request;
	union {
	struct lklm_testarg	test;
	struct lklm_lockarg	lock;
	struct lklm_unlockarg	unlock;
	} u;
};

struct lklm_reply {
	int			prot_version;
	pid_t			pid;
	int			status;
	union {
	struct lklm_holder	holder;		/* conflicting lock */
	} u;
};

#define LKLM_VERSION		0

#define LKLM_PROT_NLM1		0	/* NFSv2 file locking */
#define LKLM_PROT_NLM4		1	/* NFSv3 file locking */

#define LKLM_TEST		0
#define LKLM_LOCK		1
#define LKLM_UNLOCK		2
#define LKLM_CANCEL		3

#define LKLM_TIMEOUT		RQ_TIMEOUT
#define LKLM_GRANTED		RQ_GRANTED
#define LKLM_DENIED		RQ_DENIED
#define LKLM_DENIED_NOLOCKS	RQ_DENIED_NOLOCKS
#define LKLM_BLOCKED		RQ_BLOCKED
#define LKLM_DEADLCK		RQ_DEADLCK
#define LKLM_ROFS		RQ_ROFS
#define LKLM_STALE_FH		RQ_STALE_FH
#define LKLM_FBIG		RQ_FBIG
#define LKLM_FAILED		RQ_FAILED
#define LKLM_WORKING		32
#define LKLM_NOHOST		33

extern int	lklmfd;
void		lklm_register(void);
int		lklm_dispatch(void);

#endif /* _LKLM_H_ */
