/*
 * misc.c	Miscellaneous support functions.
 *
 * 
 */

#include "nlm.h"

/*
 * Various allocation routines. They don't do much error checking right now,
 * but you can easily replace them with e.g. the tcl memdebug functions.
 */
void *xmalloc(size_t size)
{
	void *		ptr;

	if (size == 0)
		return NULL;

	if ((ptr = malloc(size)) == NULL)
		die("malloc(%d) failed! Aborting.\n", size);
	return ptr;
}

char *xstrdup(const char *string)
{
	char	*ret;

	if ((ret = strdup(string)) == NULL)
		die("malloc(\"%s\") failed! Aborting.\n", string);
	return ret;
}

void xfree(void *ptr)
{
	free (ptr);
}

/*
 * Initialize a netobj.
 */
void
netobj_init(netobj *to, netobj *from)
{
	to->netobj_len = from->netobj_len;
	to->netobj_data = from->netobj_data;
}

/*
 * Store a netobj passed to us from RPC into our internal structure.
 */
void
netobj_copy(netobj *to, netobj *from)
{
	to->netobj_len = from->netobj_len;
	to->netobj_data = xmalloc(to->netobj_len);
	memcpy(to->netobj_data, from->netobj_data, from->netobj_len);
}

/*
 * Free the store used by a netobj
 */
void
netobj_free(netobj *obj)
{
	xfree(obj->netobj_data);
	obj->netobj_data = NULL;
	obj->netobj_len = 0;
}

/*
 * Compare to netobj's.
 */
int
netobj_cmp(netobj *ob1, netobj *ob2)
{
	if (ob1->netobj_len != ob2->netobj_len)
		return 1;
	return memcmp(ob1->netobj_data, ob2->netobj_data, ob2->netobj_len);
}

/*
 * Set a netobj to a specific value.
 */
void
netobj_set(netobj *obj, void *ptr, size_t size)
{
	obj->netobj_len = size;
	obj->netobj_data = xmalloc(size);
	if (size != 0)
		memcpy(obj->netobj_data, ptr, size);
}

/*
 * Get a netobj of a specific size.
 */
int
netobj_get(netobj *obj, void *ptr, size_t size)
{
	if (obj->netobj_len != size)
		return 0;
	if (size != 0)
		memcpy(ptr, obj->netobj_data, size);
	return 1;
}

