/*
 *	misc.h		Miscellaneous stuff. What else?
 *
 */

#ifndef LOCKD_MISC_H
#define LOCKD_MISC_H

#include <rpc/rpc.h>
#include <rpc/xdr.h>
#include <sys/time.h>

/*
 * functions for manipulations of netobj's.
 */
void		netobj_init(netobj *to, netobj *from);
void		netobj_copy(netobj *to, netobj *from);
void		netobj_free(netobj *obj);
int		netobj_cmp(netobj *obj1, netobj *obj2);
void		netobj_set(netobj *obj, void *data, size_t size);
int		netobj_get(netobj *obj, void *data, size_t size);
void		netobj_zero(netobj *obj);

#define netobj_len	n_len
#define netobj_data	n_bytes

/*
 * Dynamic memory management.
 */
void *		xmalloc(size_t size);
void *		xrealloc(void *ptr, size_t size);
char *		xstrdup(const char *str);
void		xfree(void *ptr);

/*
 * Multi-timers.
 */
typedef void	(*xtimer_fn_t)(int timerid, void * clnt_data);
int		xtimer_run(struct timeval wait, xtimer_fn_t handler,
					void * clnt_data);
int		xtimer_alarm(time_t seconds, xtimer_fn_t handler,
					void * clnt_data);
void		xtimer_stop(int timerid);

#define timeval_zero(t)		(t).tv_sec = (t).tv_usec = 0
#define timeval_set(t, sec, usec) \
		{ t.tv_sec = sec; t.tv_usec = usec; }
#define timeval_lesseq(t1, t2)		\
		((t1).tv_sec < (t2).tv_sec || \
		 ((t1).tv_sec == (t2).tv_sec && (t1).tv_usec <= (t2).tv_usec))
#define timeval_bump(t1, t2)	\
		(t1).tv_sec += (t2).tv_sec; (t1).tv_usec += (t2).tv_usec; \
		while ((t1).tv_usec >= 1000000) { \
			(t1).tv_sec++; (t1).tv_usec -= 1000000; \
		}

#endif /* LOCKD_MISC_H */
