/*
 * nlm.h	Definitions for NLMv1 and NLMv3.
 *
 *
 */

#ifndef NLM_H
#define NLM_H

#include <rpc/rpc.h>
#include "nlm_prot.h"
#include "lklm.h"

#include "rpcmisc.h"
#include "lock.h"
#include "file.h"
#include "host.h"
#include "misc.h"
#include "log.h"

/*
 * These defines are for compatibility with the X/Open NLM spec.
 */
#ifndef LOCK_GRANTED
# define LCK_GRANTED			nlm_granted
# define LCK_DENIED			nlm_denied
# define LCK_DENIED_NOLOCKS		nlm_denied_nolocks
# define LCK_BLOCKED			nlm_blocked
# define LCK_DENIED_GRACE_PERIOD	nlm_denied_grace_period
#endif

#ifndef NFS_FHSIZE
# define NFS_FHSIZE			32
#endif

/*
 * These are for debugging purposes.
 */
extern int	nlm_svcpg;		/* The RPC program # we allocate */
extern int	nlm_cltpg;		/* The NLM client RPC program # */
#define NLM_PROG_DEBUG	444444

/*
 * Externally visible protocol interface.
 */
extern lockd_proto	proto_nlm1;

/*
 * Return values for nlm_grantlock.
 */
#define GRANT_GRANTED			0
#define GRANT_FAILED			1
#define GRANT_INPROGRESS		2

/*
 * These are the main NLM server functions.
 */
void	nlm_register(int prognum);
bool_t	nlm_null_svc(struct svc_req *, void *, void *);
bool_t	nlm_test_svc(struct svc_req *, struct nlm_testargs *, nlm_testres *);
bool_t	nlm_lock_svc(struct svc_req *, struct nlm_lockargs *, nlm_res *);
bool_t	nlm_cancel_svc(struct svc_req *, struct nlm_cancargs *, nlm_res *);
bool_t	nlm_unlock_svc(struct svc_req *, struct nlm_unlockargs *, nlm_res *);
bool_t	nlm_granted_svc(struct svc_req *, struct nlm_testargs *, nlm_res *);
bool_t	nlm_test_msg_svc(struct svc_req *, struct nlm_testargs *, void *);
bool_t	nlm_lock_msg_svc(struct svc_req *, struct nlm_lockargs *, void *);
bool_t	nlm_cancel_msg_svc(struct svc_req *, struct nlm_cancargs *, void *);
bool_t	nlm_unlock_msg_svc(struct svc_req *, struct nlm_unlockargs *, void *);
bool_t	nlm_granted_msg_svc(struct svc_req *, struct nlm_testargs *, void *);
bool_t	nlm_test_res_svc(struct svc_req *, nlm_testres *, void *);
bool_t	nlm_lock_res_svc(struct svc_req *, nlm_res *, void *);
bool_t	nlm_cancel_res_svc(struct svc_req *, nlm_res *, void *);
bool_t	nlm_unlock_res_svc(struct svc_req *, nlm_res *, void *);
bool_t	nlm_granted_res_svc(struct svc_req *, nlm_res *, void *);
bool_t	nlm_share_svc(struct svc_req *, nlm_shareargs *, nlm_shareres *);
bool_t	nlm_unshare_svc(struct svc_req *, nlm_shareargs *, nlm_shareres *);
bool_t	nlm_nm_lock_svc(struct svc_req *, nlm_lockargs *, nlm_res *);
bool_t	nlm_free_all_svc(struct svc_req *, nlm_notify *, void *);

/* HACK */
typedef struct status	nsm_status;
bool_t	nlm_reclaim_svc(struct svc_req *, nsm_status *, void *);

/* Grant management */
int		nlm_grant(lockd_lock *);
void		nlm_dequeue_grantl(lockd_lock *);
lockd_lock *	nlm_dequeue_grant(int cookie);

/* Monitoring stuff */
void		nlm_monitor(lockd_host *, int);
void		nlm_unmonitor(lockd_host *);
void		nlm_reclaim_all(lockd_host *, int);
void		nlm_update_local(int action, struct lockd_req *rq);

#endif /* NLM_H */
