/*
 * nlm_dispatch.c	This file dispatches RPC calls to the NLM server.
 *
 * Cyopright (C) 1995, Olaf Kirch <okir@monad.swb.de>
 */

#include <stdio.h>
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "rpcdispatch.h"
#include "nlm.h"
#include "sm_inter.h"

static struct dispatch_entry	dtable[] = {
	table_ent(nlm_null,7,void,void),
	table_ent(nlm_test,7,nlm_testargs,nlm_testres),
	table_ent(nlm_lock,7,nlm_lockargs,nlm_res),
	table_ent(nlm_cancel,7,nlm_cancargs,nlm_res),
	table_ent(nlm_unlock,7,nlm_unlockargs,nlm_res),

	table_ent(nlm_granted,7,nlm_testargs,nlm_res),

	table_ent(nlm_test_msg,7,nlm_testargs,void),
	table_ent(nlm_lock_msg,7,nlm_lockargs,void),
	table_ent(nlm_cancel_msg,7,nlm_cancargs,void),
	table_ent(nlm_unlock_msg,7,nlm_unlockargs,void),
	table_ent(nlm_granted_msg,7,nlm_testargs,void),
	table_ent(nlm_test_res,7,nlm_testres,void),
	table_ent(nlm_lock_res,7,nlm_res,void),
	table_ent(nlm_cancel_res,7,nlm_res,void),
	table_ent(nlm_unlock_res,7,nlm_res,void),
	table_ent(nlm_granted_res,7,nlm_res,void),

	table_ent(nlm_undef,7,void,void),
	table_ent(nlm_undef,7,void,void),
	table_ent(nlm_undef,7,void,void),
	table_ent(nlm_undef,7,void,void),

	table_ent(nlm_share,4,nlm_shareargs,nlm_shareres),
	table_ent(nlm_unshare,4,nlm_shareargs,nlm_shareres),

	table_ent(nlm_nm_lock,4,nlm_lockargs,nlm_res),
	table_ent(nlm_free_all,4,nlm_notify,void),

	/* Little hack: this is where rpc.statd delivers its server crash
	 * notification to.
	 */
	table_ent(nlm_reclaim,1,status,void)
};

typedef union {
	int			null;

	struct nlm_testargs	test;
	struct nlm_lockargs	lock;
	struct nlm_cancargs	cancel;
	struct nlm_unlockargs	unlock;
	struct nlm_testargs	granted;

	struct nlm_testargs	test_msg;
	struct nlm_lockargs	lock_msg;
	struct nlm_cancargs	cancel_msg;
	struct nlm_unlockargs	unlock_msg;
	struct nlm_testargs	granted_msg;
	struct nlm_testres	test_res;
	struct nlm_res		lock_res;
	struct nlm_res		cancel_res;
	struct nlm_res		unlock_res;
	struct nlm_res		granted_res;

	struct nlm_shareargs	share;
	struct nlm_shareargs	unshare;
	struct nlm_lockargs	nm_lock;
	struct nlm_notify	free_all;

	struct status		reclaim;
} nlm_argument_union;

typedef union {
	int			voidres;
	nlm_res			res;
	nlm_testres		testres;
	nlm_shareres		shareres;
} nlm_result_union;

static void
nlm_dispatch(struct svc_req *rqstp, SVCXPRT *transp)
{
	nlm_argument_union	argument;
	nlm_result_union	result;

	rpc_dispatch(rqstp, transp,
			dtable, sizeof(dtable) / sizeof(dtable[0]),
			&argument, &result);
}


void
nlm_register(int prognum)
{
	SVCXPRT			*transp;

	if (prognum == 0)
		prognum = NLM_PROG;

	pmap_unset(prognum, 1);
	pmap_unset(prognum, 3);

	if ((transp = svcudp_create(RPC_ANYSOCK)) == NULL) {
		(void)fprintf(stderr, "cannot create udp service.\n");
		exit(1);
	}
	if (!svc_register(transp, prognum, 1, nlm_dispatch, IPPROTO_UDP)) {
		fprintf(stderr, "Unable to register (NLM, 1, udp).\n");
		exit(1);
	}
	if (!svc_register(transp, prognum, 3, nlm_dispatch, IPPROTO_UDP)) {
		fprintf(stderr, "Unable to register (NLM, 3, udp).\n");
		exit(1);
	}

	if ((transp = svctcp_create(RPC_ANYSOCK, 0, 0)) == NULL) {
		fprintf(stderr, "Cannot create tcp service.\n");
		exit(1);
	}
	if (!svc_register(transp, prognum, 1, nlm_dispatch, IPPROTO_TCP)) {
		fprintf(stderr, "Unable to register (NLM, 1, tcp).\n");
		exit(1);
	}
	if (!svc_register(transp, prognum, 3, nlm_dispatch, IPPROTO_TCP)) {
		fprintf(stderr, "Unable to register (NLM, 3, tcp).\n");
		exit(1);
	}
}
