/*
 * rcpdispatch	Generic RPC dispatcher.
 *
 * Copyright (C) 1995, Olaf Kirch <okir@monad.swb.de>
 */

#include <stdio.h>
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "rpcdispatch.h"
#include "nlm.h"
#include "lklm.h"

void
rpc_dispatch(struct svc_req *rqstp, SVCXPRT *transp,
			struct dispatch_entry *dtable, int nproc,
			void *argp, void *resp)
{
	struct dispatch_entry	*dent;

	if (rqstp->rq_proc > nproc) {
		svcerr_noproc(transp);
		return;
	}

	dent = dtable + rqstp->rq_proc;

	if (!(dent->versions & (1 << (rqstp->rq_vers - 1)))
	  || (dent->func == NULL)) {
		svcerr_noproc(transp);
		return;
	}

	memset(argp, 0, dent->xdr_arg_size);
	if (!svc_getargs(transp, dent->xdr_arg_fn, argp)) {
		svcerr_decode(transp);
		return;
	}

	log (L_DEBUG, "%s called from %s",
			dent->name, 
			inet_ntoa(svc_getcaller(rqstp->rq_xprt)->sin_addr));

	memset(resp, 0, dent->xdr_res_size);

	if ((dent->func)(rqstp, argp, resp) && resp != 0) {
		if (!svc_sendreply(transp, dent->xdr_res_fn, (caddr_t)resp)) 
			svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, dent->xdr_arg_fn, argp)) {
		log(L_ERROR, "failed to free RPC arguments");
		exit (2);
	}
}

/*
 * This is our replacement for svc_run. It turns off some signals while
 * executing the server procedures to avoid nasty race conditions.
 */
void
rpc_svcrun(void)
{
	sigset_t	block, current;
	fd_set		readfds;

	for (;;) {
		readfds = svc_fdset;
		FD_SET(lklmfd, &readfds);
		switch (select(_rpc_dtablesize(), &readfds, NULL, NULL, NULL)) {
		case -1:
			if (errno == EINTR)
				continue;
			log(L_FATAL, "svc_run: - select failed");
			break;
		case 0:
			continue;
		default:
			if (FD_ISSET(lklmfd, &readfds)) {
				lklm_dispatch();
				FD_CLR(lklmfd, &readfds);
			}
			sigemptyset(&block);
			sigaddset(&block, SIGALRM);
			sigaddset(&block, SIGVTALRM);
			sigaddset(&block, SIGIO);
			sigprocmask(SIG_BLOCK, &block, &current);
			svc_getreqset(&readfds);
			sigprocmask(SIG_SETMASK, &current, NULL);
		}
	}
}
