/*
 * rpcmisc.c	Miscellaneous RPC functions. Among other things, they
 *		implement asynchronous RPC over UDP for NLM.
 *
 * Note: rpcmsg_send and spc_send_sync should take a lockd_host as their
 * first argument instead of a struct in_addr. This way, they could cycle
 * through the list of valid addresses in the case of multiple-homed hosts
 * where one route becomes inaccessible.
 */

#ifndef RPCMISC_H
#define RPCMISC_H

int		rpccall_send(struct in_addr addr,
				int prog, int vers, int proc,
				xdrproc_t xdr_in, void *argp,
				xdrproc_t xdr_out, void *resp);
int		rpcmsg_gencookie(void);
int		rpcmsg_send(struct in_addr addr,
				int prog, int vers, int proc,
				xdrproc_t xdr_in, void *argp);
int		rpcmsg_sendreply(struct sockaddr_in *caller, int sockfd,
				u_long xid, xdrproc_t xdr_res, void *resp);
int		rpcmsg_failreply(struct sockaddr_in *caller, int sockfd,
				u_long xid, int status);
CLIENT *	rpc_getudpclnt(struct in_addr addr, int prog, int vers,
				int async);
void		rpc_killudpclient(struct in_addr addr, int prog, int vers);

/*
 * Support functions for asynchronous RPC. These functions are
 * additions to clnt_udp.c and svc_udp.c
 */
CLIENT *	clntudpa_create(struct sockaddr_in *raddr,
				u_long program, u_long version,
				struct timeval wait, int *sockp, 
				void (*callback)(CLIENT *, u_long proc,
						int cookie)
				);
u_long		svcudp_getxid(SVCXPRT *xprt);
void		svcudp_setxid(SVCXPRT *xprt, u_long xid);

/*
 * This is an addition to enum clnt_stat. It is passed to the user-suppliad
 * status report function when a callback (follow-up RPC) has been received
 * from the server.
 */
#define RPC_CALLBACK		54321

#endif /* RPCMISC_H */
